import logging

from typing import TypedDict, Optional, List, Union

from datacaster.classes import CastObject, CastDataClass

logger = logging.getLogger(__name__)


def _format_date_string(value):
    return value if not value else value.date_object.strftime("%H:%M:%S %d/%m/%Y")


def _get_decoded_ldap_string(value):
    return value.string if hasattr(value, "string") else None


class DomainController(TypedDict):
    """
- Contains Domain Controller connection details

- It is used as an _input_ to many of the actions in this plugin

- The example below shows the dictionary structure in YAML

- See the ___required___ and _optional_ keys underneath the example

!!! warning "Supplying Credentials"

    You should ___always___ use a PPA Vault integration to provide credentials to a plugin action.

!!! example "Single Address"

    ```yaml linenums="1"
    domain_controller:
      address: 1.2.3.4
      domain: example.domain.net
      port: 636
      username: [username]
      password: [password]
    ```

!!! example "Multiple Addresses For Resilience"

    ```yaml linenums="1"
    domain_controller:
      address: 1.2.3.4, 5.6.7.8
      domain: example.domain.net
      port: 636
      username: [username]
      password: [password]
    ```

!!! tip "Multiple Domain Controller Addresses"

    Version 7.6.0 of this plugin introduced support for multiple comma-separated Domain Controller addresses.

    If multiple addresses are supplied, PPA will use the first address that connects successfully.

!!! danger "Required Keys"

    __address__: Domain Controller IP or DNS address(es).

    __domain__: FQDN of the Active Directory domain.

    __username__: Username for authentication.

    __password__: Password for authentication.

!!! info "Optional Keys"

    __port__: The LDAPS port on the Domain Controller (defaults to `636`).

    """

    address: str
    port: Optional[int]
    domain: str
    username: str
    password: str


class PasswordExpiry(TypedDict):
    """
- Contains user password expiry information

!!! info "PasswordExpiry Keys"

    __total_seconds__: Total number of seconds until expiry.

    __timestamp__: Expiry date as a timestamp in the following format:

    ```
      13:26:30 30/11/2021 UTC
    ```

    __days__: Number of days.

    __hours__: Number of hours.

    __minutes__: Number of minutes.

    __seconds__: Number of seconds.


!!! important "Time Until Expiry"

    The values of **days**, **hours**, **minutes**, & **seconds** are not _totals_.

    They are a _portion_ of the remaining time until expiry.

    For example, the value of **hours** is not the total number of hours until expiry.

    Instead it should be used in conjunction with the other keys to display a countdown:

    {% raw %}
    ```YAML linenums="1"
    - ppa.ui.output_markdown:
        doc: >
          Your password will expire in:

          **Days:** {{ expiry.days }}

          **Hours:** {{ expiry.hours }}

          **Minutes:** {{ expiry.minutes }}
    ```
    {% endraw %}

    As **days** is the largest unit, it can be used on its own to find accounts close to expiry:

    {% raw %}
    ```YAML linenums="1"
    - ppa.ui.output_error:
        text: Your password will expire in {{ expiry.days }} days.
      when: expiry.days < 7
    ```
    {% endraw %}
    """

    days: int
    hours: int
    minutes: int
    seconds: int
    timestamp: str
    total_seconds: int


class TimePeriod(TypedDict):
    """
- Describes a period of time

!!! info "TimePeriod Keys"

    __days__: Number of days.

    __hours__: Number of hours.

    __minutes__: Number of minutes.

    __seconds__: Number of seconds.


!!! important "Using Values"

    The values in this dictionary are not _totals_, they are a _portion_ of the time period.

    For example, the value of **hours** is not the total number of hours.

    Instead it should be used in conjunction with the other keys to display a total:

    {% raw %}
    ```YAML linenums="1"
    - ppa.ui.output_markdown:
        doc: >
          The maximum password age is:

          **Days:** {{ expiry.days }}

          **Hours:** {{ expiry.hours }}

          **Minutes:** {{ expiry.minutes }}
    ```
    {% endraw %}
    """

    days: int
    hours: int
    minutes: int
    seconds: int


class User(TypedDict):
    """
- Contains Active Directory User LDAP attributes

- Expand the sections below for more information

??? info "Standard Keys"

    __accountExpires__: Date string or null [[Account-Expires](https://docs.microsoft.com/en-us/windows/win32/adschema/a-accountexpires)].

    __badPasswordTime__: Date string or null [[Bad-Password-Time](https://docs.microsoft.com/en-us/windows/win32/adschema/a-badpasswordtime)].

    __badPwdCount__: Integer [[Bad-Pwd-Count](https://docs.microsoft.com/en-us/windows/win32/adschema/a-badpwdcount)].

    __carLicense__: String or null [[carLicense](https://docs.microsoft.com/en-us/windows/win32/adschema/a-carlicense)].

    __cn__: String or null [[Common-Name](https://docs.microsoft.com/en-us/windows/win32/adschema/a-cn)].

    __company__: String or null [[Company](https://docs.microsoft.com/en-us/windows/win32/adschema/a-company)].

    __countryCode__: String or null [[Country-Code](https://docs.microsoft.com/en-us/windows/win32/adschema/a-countrycode)].

    __description__: String or null [[Description](https://docs.microsoft.com/en-us/windows/win32/adschema/a-description)].

    __displayName__: String or null [[Display-Name](https://docs.microsoft.com/en-us/windows/win32/adschema/a-displayname)].

    __distinguishedName__: String [[Obj-Dist-Name](https://docs.microsoft.com/en-us/windows/win32/adschema/a-distinguishedname)].

    __employeeNumber__: String [[Employee-Number](https://docs.microsoft.com/en-us/windows/win32/adschema/a-employeenumber)].

    __gidNumber__: Integer or null [[gidNumber](https://docs.microsoft.com/en-us/windows/win32/adschema/a-gidNumber)].

    __givenName__: String or null [[Given-Name](https://docs.microsoft.com/en-us/windows/win32/adschema/a-givenname)].

    __info__: String or null [[Comment](https://docs.microsoft.com/en-us/windows/win32/adschema/a-info)].

    __l__: String or null [[Locality-Name](https://docs.microsoft.com/en-gb/windows/win32/adschema/a-l)].

    __lastLogoff__: Date string or null [[Last-Logoff](https://docs.microsoft.com/en-us/windows/win32/adschema/a-lastlogoff)].

    __lastLogon__: Date string or null [[Last-Logon](https://docs.microsoft.com/en-us/windows/win32/adschema/a-lastlogon)].

    __lastLogonTimestamp__: Date string or null [[Last-Logon-Timestamp](https://docs.microsoft.com/en-us/windows/win32/adschema/a-lastlogontimestamp)].

    __lockoutTime__: Date string or null [[Lockout-Time](https://docs.microsoft.com/en-us/windows/win32/adschema/a-lockouttime)].

    __loginShell__: String or null [[loginShell](https://docs.microsoft.com/en-us/windows/win32/adschema/a-loginShell)].

    __logonCount__: Integer [[Logon-Count](https://docs.microsoft.com/en-us/windows/win32/adschema/a-logoncount)].

    __mail__: String or null [[E-mail-Addresses](https://docs.microsoft.com/en-us/windows/win32/adschema/a-mail)].

    __manager__: String or null [[Manager](https://docs.microsoft.com/en-us/windows/win32/adschema/a-manager)].

    __memberOf__: List of [Group Dictionaries](#group) [[Is-Member-Of-DL](https://docs.microsoft.com/en-us/windows/win32/adschema/a-memberof)].

    __name__: String or null [[RDN](https://docs.microsoft.com/en-us/windows/win32/adschema/a-name)].

    __objectGUID__: String [[Object-Guid](https://docs.microsoft.com/en-us/windows/win32/adschema/a-objectguid)].

    __objectSid__: String [[Object-Sid](https://docs.microsoft.com/en-us/windows/win32/adschema/a-objectsid)].

    __postalCode__: String or null [[Postal-Code](https://docs.microsoft.com/en-gb/windows/win32/adschema/a-postalcode)].

    __physicalDeliveryOfficeName__: String or null [[Physical-Delivery-Office-Name](https://docs.microsoft.com/en-gb/windows/win32/adschema/a-physicaldeliveryofficename)].

    __pwdLastSet__: Date string or null [[Pwd-Last-Set](https://docs.microsoft.com/en-us/windows/win32/adschema/a-pwdlastset)].

    __sAMAccountName__: String [[Sam-Account-Name](https://docs.microsoft.com/en-us/windows/win32/adschema/a-samaccountname)].

    __sAMAccountType__: Integer [[Sam-Account-Type](https://docs.microsoft.com/en-us/windows/win32/adschema/a-samaccounttype)].

    __sn__: String or null [[Surname](https://docs.microsoft.com/en-us/windows/win32/adschema/a-sn)].

    __street__: String or null [despite the name this is the [Street-Address](https://docs.microsoft.com/en-gb/windows/win32/adschema/a-street)].

    __streetAddress__: String or null [despite the name this is the [Address](https://docs.microsoft.com/en-gb/windows/win32/adschema/a-streetaddress)].

    __telephoneNumber__: String or null [[Telephone-Number](https://docs.microsoft.com/en-us/windows/win32/adschema/a-telephonenumber)].

    __thumbnailPhoto__: String or null [[Picture](https://docs.microsoft.com/en-us/windows/win32/adschema/a-thumbnailphoto)].

    __uid__: String or null [[uid](https://docs.microsoft.com/en-us/windows/win32/adschema/a-uid)].

    __uidNumber__: String or null [[uidNumber](https://docs.microsoft.com/en-us/windows/win32/adschema/a-uidNumber)].

    __unixHomeDirectory__: String or null [[unixHomeDirectory](https://docs.microsoft.com/en-gb/windows/win32/adschema/a-unixHomeDirectory)].

    __userAccountControl__: Integer [[User-Account-Control](https://docs.microsoft.com/en-us/windows/win32/adschema/a-useraccountcontrol)].

    __userPrincipalName__: String or null [[User-Principal-Name](https://docs.microsoft.com/en-us/windows/win32/adschema/a-userprincipalname)].

??? info "Extra Keys"

    The following keys are created by PPA based on other Active Directory attributes:

    __is_disabled__: Boolean indicating whether the user is disabled.

    __is_enabled__: Boolean indicating whether the user is enabled.

    __is_expired__: Boolean indicating whether the user is expired.

    __is_locked__: Boolean indicating whether the user is locked out.

    __password_age__: Either a dictionary containing the user's password age in:

        - days
        - hours
        - minutes
        - seconds

        Or `null` if no password has been set.


!!! tip "See more about Users [here](https://docs.microsoft.com/en-gb/windows/win32/adschema/c-user)"

    """

    accountExpires: str
    badPasswordTime: str
    badPwdCount: Optional[int]
    carLicense: Optional[List[str]]
    cn: Optional[str]
    company: Optional[str]
    countryCode: Optional[str]
    description: Optional[str]
    displayName: Optional[str]
    distinguishedName: str
    employeeNumber: Optional[str]
    gidNumber: Optional[int]
    givenName: Optional[str]
    info: Optional[str]
    is_disabled: bool
    is_enabled: bool
    is_expired: bool
    is_locked: bool
    l: Optional[str]
    lastLogoff: str
    lastLogon: str
    lastLogonTimestamp: str
    lockoutTime: str
    loginShell: Optional[str]
    logonCount: Optional[int]
    mail: Optional[str]
    manager: Optional[str]
    memberOf: Optional[List[str]]
    name: Optional[str]
    objectGUID: str
    objectSid: str
    password_age: Optional[TimePeriod] = None
    postalCode: Optional[str]
    physicalDeliveryOfficeName: Optional[str]
    pwdLastSet: str
    sAMAccountName: str
    sAMAccountType: Optional[int]
    sn: Optional[str]
    street: Optional[str]
    streetAddress: Optional[str]
    telephoneNumber: Optional[str]
    thumbnailPhoto: Optional[str]
    uid: Optional[str]
    uidNumber: Optional[int]
    unixHomeDirectory: Optional[str]
    userAccountControl: int
    userPrincipalName: Optional[str]


class UserData(CastObject):
    __annotations__ = User.__dict__["__annotations__"]
    __class_config__ = {
        "cast_functions": {
            "fields": {
                "objectGUID": _get_decoded_ldap_string,
                "objectSid": _get_decoded_ldap_string,
                "thumbnailPhoto": _get_decoded_ldap_string,
                "accountExpires": _format_date_string,
                "badPasswordTime": _format_date_string,
                "lastLogon": _format_date_string,
                "lastLogoff": _format_date_string,
                "lastLogonTimestamp": _format_date_string,
                "lockoutTime": _format_date_string,
                "pwdLastSet": _format_date_string,
            },
        },
        "always_cast": ["objectGUID", "objectSid", "thumbnailPhoto"],
    }


class Group(TypedDict):
    """
- Contains Active Directory Group LDAP attributes

- Expand below to see each key and a link to the relevant Microsoft article

??? info "Standard Keys"

    __cn__: String or null [[Common-Name](https://docs.microsoft.com/en-us/windows/win32/adschema/a-cn)].

    __description__: String or null [[Description](https://docs.microsoft.com/en-us/windows/win32/adschema/a-description)].

    __distinguishedName__: String [[Obj-Dist-Name](https://docs.microsoft.com/en-us/windows/win32/adschema/a-distinguishedname)].

    __gidNumber__: Integer or null [[gidNumber](https://docs.microsoft.com/en-us/windows/win32/adschema/a-gidNumber)].

    __groupType__: String [[Group-Type](https://docs.microsoft.com/en-gb/windows/win32/adschema/a-grouptype)].

    __info__: String or null [[Comment](https://docs.microsoft.com/en-us/windows/win32/adschema/a-info)].

    __managedBy__: String or null [[ManagedBy](https://docs.microsoft.com/en-us/windows/win32/adschema/a-managedby)].

    __member__: List of [User](#user), [Group](#group), or [Computer](#computer) distinguished names or null [[Member](https://docs.microsoft.com/en-us/windows/win32/adschema/a-member)].

    __memberUid__: List of strings or null [[memberUid](https://docs.microsoft.com/en-us/windows/win32/adschema/a-memberuid)].

    __name__: String or null [[RDN](https://docs.microsoft.com/en-us/windows/win32/adschema/a-name)].

    __objectGUID__: String [[Object-Guid](https://docs.microsoft.com/en-us/windows/win32/adschema/a-objectguid)].

    __objectSid__: String [[Object-Sid](https://docs.microsoft.com/en-us/windows/win32/adschema/a-objectsid)].

    __sAMAccountName__: String [[Sam-Account-Name](https://docs.microsoft.com/en-us/windows/win32/adschema/a-samaccountname)].

    __sAMAccountType__: Integer [[Sam-Account-Type](https://docs.microsoft.com/en-us/windows/win32/adschema/a-samaccounttype)].

!!! warning "Group Membership Limit"

    Due to an Active Directory limitation, the __member__ key can only hold up to 1500 group members.

    If the group has more than 1500 members, only members 1-1500 will be included in this key.

    You can bypass this limitation by getting _nested_ users with the [get_members](../active_directory.groups/#get_members) action.


!!! tip "See more about Groups [here](https://docs.microsoft.com/en-gb/windows/win32/adschema/c-group)"

    """

    cn: Optional[str]
    description: Optional[str]
    distinguishedName: str
    gidNumber: Optional[int]
    groupType: str
    info: Optional[str]
    managedBy: Optional[str]
    member: Optional[List[str]]
    memberUid: Optional[List[str]]
    name: Optional[str]
    objectGUID: str
    objectSid: str
    sAMAccountName: str
    sAMAccountType: int


class GroupData(CastDataClass):
    __annotations__ = Group.__dict__["__annotations__"]
    __class_config__ = {
        "cast_functions": {"fields": {"objectGUID": _get_decoded_ldap_string, "objectSid": _get_decoded_ldap_string}},
        "always_cast": ["objectGUID", "objectSid"],
    }


class OrganizationalUnit(TypedDict):
    """
- Contains Active Directory Organizational Unit LDAP attributes

- Expand below to see each key and a link to the relevant Microsoft article

??? info "Standard Keys"

    __distinguishedName__: String [[Obj-Dist-Name](https://docs.microsoft.com/en-us/windows/win32/adschema/a-distinguishedname)].

    __name__: String [[Organizational-Unit-Name](https://docs.microsoft.com/en-us/windows/win32/adschema/a-ou)].

    __objectGUID__: String [[Object-Guid](https://docs.microsoft.com/en-us/windows/win32/adschema/a-objectguid)].

    __ou__: String [[Organizational-Unit-Name](https://docs.microsoft.com/en-us/windows/win32/adschema/a-ou)].

!!! tip "See more about Organizational Units [here](https://docs.microsoft.com/en-gb/windows/win32/adschema/c-organizationalunit)"

    """

    distinguishedName: str
    name: Optional[str]
    objectGUID: str
    ou: Optional[str]


class OrganizationalUnitData(CastObject):
    __annotations__ = OrganizationalUnit.__dict__["__annotations__"]
    __class_config__ = {
        "cast_functions": {"fields": {"objectGUID": _get_decoded_ldap_string}},
        "always_cast": ["objectGUID"],
    }


class Container(TypedDict):
    """
- Contains Active Directory Container LDAP attributes

- Expand below to see each key and a link to the relevant Microsoft article

??? info "Standard Keys"

    __description__: String or null [[Description](https://docs.microsoft.com/en-us/windows/win32/adschema/a-description)].

    __distinguishedName__: String [[Obj-Dist-Name](https://docs.microsoft.com/en-us/windows/win32/adschema/a-distinguishedname)].

    __name__: String or null [[RDN](https://docs.microsoft.com/en-us/windows/win32/adschema/a-name)].

    __cn__: String or null [[Common-Name](https://docs.microsoft.com/en-us/windows/win32/adschema/a-cn)].

!!! tip "See more about Containers [here](https://docs.microsoft.com/en-gb/windows/win32/adschema/c-container)"

    """

    description: Optional[str]
    distinguishedName: str
    name: Optional[str]
    cn: Optional[str]


class ContainerData(CastObject):
    __annotations__ = Container.__dict__["__annotations__"]
    __class_config__ = {
        "cast_functions": {"fields": {"objectGUID": _get_decoded_ldap_string}},
        "always_cast": ["objectGUID"],
    }


class Computer(TypedDict):
    """
- Contains Active Directory Computer LDAP attributes

- Expand below to see each key and a link to the relevant Microsoft article

??? info "Standard Keys"

    __accountExpires__: Date string or null [[Account-Expires](https://docs.microsoft.com/en-us/windows/win32/adschema/a-accountexpires)].

    __carLicense__: String or null [[carLicense](https://docs.microsoft.com/en-us/windows/win32/adschema/a-carlicense)].

    __cn__: String or null [[Common-Name](https://docs.microsoft.com/en-us/windows/win32/adschema/a-cn)].

    __description__: String or null [[Description](https://docs.microsoft.com/en-us/windows/win32/adschema/a-description)].

    __displayName__: String or null [[Display-Name](https://docs.microsoft.com/en-us/windows/win32/adschema/a-displayname)].

    __distinguishedName__: String [[Obj-Dist-Name](https://docs.microsoft.com/en-us/windows/win32/adschema/a-distinguishedname)].

    __dNSHostName__: String [[DNS-Host-Name](https://docs.microsoft.com/en-gb/windows/win32/adschema/a-dnshostname)].

    __info__: String or null [[Comment](https://docs.microsoft.com/en-us/windows/win32/adschema/a-info)].

    __lastLogoff__: Date string or null [[Last-Logoff](https://docs.microsoft.com/en-us/windows/win32/adschema/a-lastlogoff)].

    __lastLogon__: Date string or null [[Last-Logon](https://docs.microsoft.com/en-us/windows/win32/adschema/a-lastlogon)].

    __lastLogonTimestamp__: Date string or null [[Last-Logon-Timestamp](https://docs.microsoft.com/en-us/windows/win32/adschema/a-lastlogontimestamp)].

    __lockoutTime__: Date string or null [[Lockout-Time](https://docs.microsoft.com/en-us/windows/win32/adschema/a-lockouttime)].

    __logonCount__: Integer [[Logon-Count](https://docs.microsoft.com/en-us/windows/win32/adschema/a-logoncount)].

    __mail__: String or null [[E-mail-Addresses](https://docs.microsoft.com/en-us/windows/win32/adschema/a-mail)].

    __mail__: String or null [[Managed-By](https://docs.microsoft.com/en-us/windows/win32/adschema/a-managedby)].

    __memberOf__: List of [Group Dictionaries](#group) [[Is-Member-Of-DL](https://docs.microsoft.com/en-us/windows/win32/adschema/a-memberof)].

    __name__: String or null [[RDN](https://docs.microsoft.com/en-us/windows/win32/adschema/a-name)].

    __objectGUID__: String [[Object-Guid](https://docs.microsoft.com/en-us/windows/win32/adschema/a-objectguid)].

    __objectSid__: String [[Object-Sid](https://docs.microsoft.com/en-us/windows/win32/adschema/a-objectsid)].

    __operatingSystem__: String or null [[Operating-System](https://docs.microsoft.com/en-us/windows/win32/adschema/a-operatingsystem)].

    __operatingSystemHotfix__: String or null [[Operating-System-Hotfix](https://docs.microsoft.com/en-us/windows/win32/adschema/a-operatingsystemhotfix)].

    __operatingSystemServicePack__: String or null [[Operating-System-Service-Pack](https://docs.microsoft.com/en-us/windows/win32/adschema/a-operatingsystemservicepack)].

    __operatingSystemVersion__: String or null [[Operating-System-Version](https://docs.microsoft.com/en-us/windows/win32/adschema/a-operatingsystemversion)].

    __pwdLastSet__: Date string or null [[Pwd-Last-Set](https://docs.microsoft.com/en-us/windows/win32/adschema/a-pwdlastset)].

    __sAMAccountName__: String [[Sam-Account-Name](https://docs.microsoft.com/en-us/windows/win32/adschema/a-samaccountname)].

    __sAMAccountType__: Integer [[Sam-Account-Type](https://docs.microsoft.com/en-us/windows/win32/adschema/a-samaccounttype)].

    __userAccountControl__: Integer [[User-Account-Control](https://docs.microsoft.com/en-us/windows/win32/adschema/a-useraccountcontrol)].

    __userPrincipalName__: String or null [[User-Principal-Name](https://docs.microsoft.com/en-us/windows/win32/adschema/a-userprincipalname)].

!!! tip "See more about Computers [here](https://docs.microsoft.com/en-gb/windows/win32/adschema/c-computer)"
    """

    accountExpires: str
    carLicense: Optional[List[str]]
    cn: Optional[str]
    description: Optional[str]
    displayName: Optional[str]
    distinguishedName: str
    dNSHostName: Optional[str]
    info: Optional[str]
    lastLogoff: Optional[str]
    lastLogon: str
    lastLogonTimestamp: str
    lockoutTime: str
    logonCount: Optional[int]
    mail: Optional[str]
    managedBy: Optional[str]
    memberOf: Optional[List[str]]
    name: Optional[str]
    objectGUID: str
    objectSid: str
    operatingSystem: Optional[str]
    operatingSystemHotfix: Optional[str]
    operatingSystemServicePack: Optional[str]
    operatingSystemVersion: Optional[str]
    pwdLastSet: str
    sAMAccountName: str
    sAMAccountType: int
    userAccountControl: int
    userPrincipalName: Optional[str]


class ComputerData(CastObject):
    __annotations__ = Computer.__dict__["__annotations__"]
    __class_config__ = {
        "cast_functions": {
            "fields": {
                "objectGUID": _get_decoded_ldap_string,
                "objectSid": _get_decoded_ldap_string,
                "accountExpires": _format_date_string,
                "lastLogon": _format_date_string,
                "lastLogoff": _format_date_string,
                "lastLogonTimestamp": _format_date_string,
                "lockoutTime": _format_date_string,
                "pwdLastSet": _format_date_string,
            },
        },
        "always_cast": ["objectGUID", "objectSid"],
    }


UACPropertyFlags = {
    "SCRIPT": 1,
    "ACCOUNTDISABLE": 2,
    "HOMEDIR_REQUIRED": 8,
    "LOCKOUT": 16,
    "PASSWD_NOTREQD": 32,
    "PASSWD_CANT_CHANGE": 64,
    "ENCRYPTED_TEXT_PWD_ALLOWED": 128,
    "TEMP_DUPLICATE_ACCOUNT": 256,
    "NORMAL_ACCOUNT": 512,
    "INTERDOMAIN_TRUST_ACCOUNT": 2048,
    "WORKSTATION_TRUST_ACCOUNT": 4096,
    "SERVER_TRUST_ACCOUNT": 8192,
    "DONT_EXPIRE_PASSWORD": 65536,
    "MNS_LOGON_ACCOUNT": 131072,
    "SMARTCARD_REQUIRED": 262144,
    "TRUSTED_FOR_DELEGATION": 524288,
    "NOT_DELEGATED": 1048576,
    "USE_DES_KEY_ONLY": 2097152,
    "DONT_REQ_PREAUTH": 4194304,
    "PASSWORD_EXPIRED": 8388608,
    "TRUSTED_TO_AUTH_FOR_DELEGATION": 16777216,
    "PARTIAL_SECRETS_ACCOUNT": 67108864,
}


class AttributePair(TypedDict):
    """
- Contains an LDAP attribute **name** & **value**

??? info "Standard Keys"

    __name__: The name of the LDAP attribute.

    __value__: A value to set for the attribute.
    """

    name: str
    value: Union[str, int, float, None, bool]


AttributeSet = List[AttributePair]
